/**
 * Copyright (c) 2008 Barcelona Supercomputing Center - Centro Nacional de
 * Supercomputacion, Heinrich-Heine-Universitaet Duesseldorf, and XLAB d.o.o.
 * All rights reserved.
 *
 * This file is part of DIXI. DIXI is part of XtreemOS, a Linux-based Grid
 * Operating System, see <http://www.xtreemos.eu> for more details. The
 * XtreemOS project has been developed with the financial support of the
 * European Commission's IST program under contract #FP6-033576.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the XtreemOS project nor the names of its
 *     partners and/or contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef XATICATYPES_H
#define XATICATYPES_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/**
 *  Enumerations for XATICA parameters, note that XOS_ prefix has been adopted
 *
 */
enum TypeOfInfo {
  XOS_BASIC=2,
  XOS_JOB_DEFINITION=4,
  XOS_RESOURCES_ALLOCATED=8,
  XOS_RESOURCES_CONSUMED=16,
  XOS_USER_METRICS=32,
  XOS_NO_BUFFER=64,
  XOS_XTRACE=128,
  XOS_ENABLE=256
};

enum InfoLevel {
  XOS_JOB=1,
  XOS_PROCESS,
  XOS_KERNEL
};

enum DependenceDirection {
  XOS_UP = 0,
  XOS_DOWN
};

enum ControlOperations {
  XOS_SUSPENDJOB=19,
  XOS_RESUMEJOB=18,
  XOS_CANCELJOB=15
};

enum MetricType {
  XOS_MTINT,
  XOS_MTDOUBLE,
  XOS_MTCHAR,
  XOS_MTSTRING,
  XOS_MTTIME
};

enum MetricScope {
  XOS_MSJOB,
  XOS_MSRESOURCE,
  XOS_MSJOBUNIT,
  XOS_MSPROCESS,
};

enum MetricBufSize {
  XOS_MBSHORT,
  XOS_MBMEDIUM,
  XOS_MBLONG,
};

enum JobHints {
	XOS_JHMIGRATION_REQ = 0,
	XOS_JHSCHEDULING_REQ = 1
};

enum SchedulingHints {
	XOS_SHDEFAULT=0,
	XOS_SHONEPERNODE=1
};

/* XATICA parameters: ArrayList, CommunicationAddress and MetricsDesc */
struct arrayList
{
  char **list;
  int size;
};
typedef struct arrayList* ArrayList;

struct communicationAddress{
  struct in_addr host;
  struct in_addr *externalAddress;
  unsigned short int port;
};
typedef struct communicationAddress* CommunicationAddress;

struct arrayListCA
{
  CommunicationAddress *list;
  int size;
};
typedef struct arrayListCA* ArrayListCA;

struct metricsDesc
{
  char *name;
  enum MetricType type;
  char *description;
  enum MetricScope scope;
  int bufferable;
  enum MetricBufSize bufsize;
};
typedef struct metricsDesc* MetricsDesc;

/**
 *  Helper structures to parse XML output
 *  These have constructors/destructors (new or XML2/free) and
 *  must be used through the specified functions in this file.
 *
 *  -=For example=-
 *
 *  JobInfoList myJil;
 *  ArrayList jobs;
 *
 *  myJil = XML2JobInfoList(jInfoXML);
 *  if (myJil == NULL){
 *    fprintf(stderr, "parse error\n");
 *    return -1;
 *  }
 *
 *  int errcode = getJobs(jil, &jobs);
 */
typedef struct
{
  unsigned long long timestamp;
  char *value;
} TimedValue;

typedef struct
{
  TimedValue **list;
  int size;
} ArrayListTV;

struct metricValue
{
  char *name;
  enum MetricType type;
  ArrayListTV values;
};
typedef struct metricValue* MetricValue;

typedef struct{
  void **list;
  int size;
  void **list2;
  int size2;
  CommunicationAddress resourceId;
  int pid;
} ArrayListXM;

typedef struct{
  char *jobId;
  ArrayListXM jMetrics;
  ArrayListXM rMetrics;
  ArrayListXM metrics;
}JobInfo;

struct jobInfoList
{
  JobInfo **list;
  int size;
};
typedef struct jobInfoList* JobInfoList;

struct arrayListInt
{
  int *list;
  int size;
};
typedef struct arrayListInt* ArrayListInt;

/* ReservationRequest */
struct reservationRequest
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct reservationRequest* ReservationRequest;
struct arrayListReservationRequest
{
	ReservationRequest *request;
	int size;
};
typedef struct arrayListReservationRequest* ArrayListReservationRequest;

/* UpdateRequest */
struct updateRequest
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct updateRequest* UpdateRequest;
struct arrayListUpdateRequest
{
	UpdateRequest *request;
	int size;
};
typedef struct arrayListUpdateRequest* ArrayListUpdateRequest;

/* ReservationSlot */
struct reservationSlot
{
	CommunicationAddress *nodeAddress;
	char **localRequest;
};
typedef struct reservationSlot* ReservationSlot;
struct arrayListReservationSlot
{
	ReservationSlot *request;
	int size;
};
typedef struct arrayListReservationSlot* ArrayListReservationSlot;

/* XATICA parameters' functions */
/* ArrayList */
ArrayList newArrayList();
void freeArrayList(ArrayList array);
int addToArrayList(ArrayList array, char *el);
char *getFromArrayList(ArrayList array, int elno);
char *ArrayList2XML(ArrayList array);

/* CommunicationAddress */
int CommAddress2str(const CommunicationAddress ca, char **str);
int str2CommAddress(const char *str, CommunicationAddress *ca);
int equalsCommAddress(const CommunicationAddress ca1, const CommunicationAddress ca2);
void freeCommAddress(CommunicationAddress ca1);

/* MetricsDesc */
MetricsDesc newMetricsDesc();
MetricsDesc XML2MetricsDesc(const char *xmlin);
char *MetricsDesc2XML(const MetricsDesc md);
void setMetricsDescName(MetricsDesc md, const char *name);
void setMetricsDescDescription(MetricsDesc md, const char *description);
void setMetricsDescScope(MetricsDesc md, enum MetricScope ms);
void setMetricsDescType(MetricsDesc md, enum MetricType mt);
void setMetricsDescBufferable(MetricsDesc md, int bufferable);
void setMetricsDescBufSize(MetricsDesc md, enum MetricBufSize mb);
void freeMetricsDesc(MetricsDesc md);

/* Helper structures' functions */
/* JobInfoList*/
/**
 * This function constructs a JobInfoList with the contents of a XML String
 * conforming to the JobInfoList Schema. The whole tree is stored in memory
 * so there is also a function to free it.
 * @param xmlin a string with the XML message received from getJobsInfo
 * @return NULL if XML doesn't validate.
 * @see #freeJobInfoList(JobInfoList jil)
 * FIXME add @deprecated offers no XSD validation
 */
JobInfoList XML2JobInfoList(const char *xmlin);

/**FIXME: Uncomment when xmlSchemaValidateDoc works.
 * This function constructs a JobInfoList with the contents of a XML String
 * conforming to the JobInfoList Schema. The whole tree is stored in memory
 * so there is also a function to free it.
 * @param xmlin a string with the XML message received from getJobsInfo
 * @return NULL if XML doesn't validate.
 * @see #freeJobInfoList(JobInfoList jil)
 */
//JobInfoList xml2JobInfoList(const char *xmlin, const char *schemasLocation);

void freeJobInfoList(JobInfoList jil);
int getJobs(const JobInfoList jil, ArrayList *array);
int getJobResources(const JobInfoList jil, const char *jobId, ArrayListCA *array);
int getResourcePids(const JobInfoList jil, const char *jobId, const CommunicationAddress ca, ArrayListInt *array);

/* MetricValue*/
/**
 * This function searches for a specific metric in the JobInfoList
 * structure by its name, scope, resourceId and pid when needed.
 * Then it sets the contents of a MetricValue structure with its
 * value/s (several when buffered).
 * @see #MetricValue
 * @param jil a pointer to the JobInfoList that contains the answer
 *            of getJobsInfo.
 * @see #XML2JobInfoList(const char *xmlin, JobInfoList *jil)
 * @param scope an enum MetricScope specifying the scope of the
 *        requested metric.
 * @param resourceId a CommunicationAddress specifying
 *        the resource, the requested metric belongs to. It can be
 *        NULL when not applicable: Job scope.
 * @param pid an integer specifying the process id of the process,
 *        the requested metric belongs to. It can be 0 when not
 *        applicable: Job/JobUnit/Resource scope.
 * @param metricName a string specifying the name of the metric.
 * @param mv a pointer to a MetricValue structure that will be set
 *        with the values from the metric.
 * @return error codes
 */
int getMetricValue(const JobInfoList jil, enum MetricScope scope, const char *jobId, const CommunicationAddress resourceId, const int pid, const char *metricName, MetricValue *mv);

void freeMetricValue(MetricValue mv);

/**
 * This function returns the first value present in the MetricValue
 * structure. It's the most common operation when using non-buffered
 * metrics.
 *
 * @param mv a pointer to the MetricValue struct.
 * @return a string with the first value of the MetricValue struct.
 */
char *getFirstMetricValue(const MetricValue mv);

char *getFromMetricValue(const MetricValue mv, int elno);
unsigned long long getTimestampFromMetricValue(const MetricValue mv, int elno);
int getMetricValueLength(const MetricValue mv);

/* ArrayListCA */
ArrayListCA newArrayListCA();
int addToArrayListCA(ArrayListCA array, CommunicationAddress el);
CommunicationAddress getFromArrayListCA(ArrayListCA array, int elno);
char *ArrayListCA2XML(ArrayListCA array);
void freeArrayListCA(ArrayListCA array);

/* ArrayListInt */
ArrayListInt newArrayListInt();
int addToArrayListInt(ArrayListInt array, int el);
int getFromArrayListInt(ArrayListInt array, int elno);
void freeArrayListInt(ArrayListInt array);

/* ReservationRequest */
ReservationRequest *ReservationRequest2XML(char *xml);
char *XML2ReservationRequest(ReservationRequest request);
ArrayListReservationRequest XML2ArrayListReservationRequest(char *xml);
char *ArrayListReservationRequest2XML(ArrayListReservationRequest list);

/* UpdateRequest */
UpdateRequest *UpdateRequest2XML(char *xml);
char *XML2UpdateRequest(UpdateRequest request);
ArrayListUpdateRequest XML2ArrayListUpdateRequest(char *xml);
char *ArrayListUpdateRequest2XML(ArrayListUpdateRequest list);

/* ReservationSlot */
ReservationSlot *ReservationSlot2XML(char *xml);
char *XML2ReservationSlot(ReservationSlot slot);
ArrayListReservationSlot XML2ArrayListReservationSlot(char *xml);
char *ArrayListReservationSlot2XML(ArrayListReservationSlot list);


#endif
 #ifdef __cplusplus
 }
 #endif
