#include "XATICATypes.h"
#ifdef __cplusplus
 extern "C" {
 #endif
	/**
	 * Adds certificate passed as an argument into a list of trusted certificates 
	 * (VO admins list). This method is used as a base of access control.
	 * 
	 * @param voAdminsCert certificate to add into a list of trusted certificates.
	 * See also {@link X509Certificate}.
	 * @return true on success.
	 */
int registerVoAdmin(const char* __voAdminsCert, char* returnValue);


	/**
	 * Removes certificate with specified index from a list of trusted
	 * certificates. 
	 * 
	 * @param voAdminsCert
	 * @return true on success
	 */
int unregisterVoAdmin(int __index, char* returnValue);


	/**
	 * Lists registered VO administrators.
	 * 
	 * @return list in a instance of String. 
	 */
int listVoAdmins( char** returnValue);



int obtainFilterPolicyAEM(const char* __jsdlContent,const char* __action, char** returnValue);


	/**
	 * Used by AEM framework to check if resources listed in 
	 * {@list ResourceMatching} comply with policies stored in VO policy storage
	 * {@link PolicyFactory#listPolicies()}.  
	 * 
	 * @param xos_cert instance of {@link X509Certificate}.
	 * @param resources list of potential compliant resource, see also 
	 * ResourceMathching class.
	 * @param action String used to denote the action taken by the user over 
	 * resources. Use constants declared inside {@link XACMLConstants#Action}. 
	 * @return instance of ResourceMatching, also includes digital signature, 
	 * see {@link ResourceMatching#setSignature(byte[])}and
	 * {@link ResourceMatching#getSignature()}.
	 */
int verifyPolicyAEM(const char* __action, void* returnValue);


	/**
	 * This method is called by consequence of the 
	 * {@link VOPS#verifyPolicyAEM(Object, ResourceMatching, String)} method.
	 * Enforces policies - generates XACML request for each of the resource
	 * and checks it against policies residing in policy storage, see 
	 * {@link PolicyFactory#listPolicies()}.
	 *  
	 * @param xos_cert User certificate which has been stored in VOPSStorage
	 * @param resources are collection from process of resource mathcing, see 
	 * {@link ResourceMatching}. 
	 * @return instance of the ResourceMatching object. It contains a list of 
	 * all potential resource nodes.
	 */
int verifyPolicyCertRes(const char* __xos_cert, void* returnValue);


	/**
	 * A new policy file is created in policy storage using static PolicyFactory class. Policy has
	 * policyID, description. 
	 *
	 * @param policyID Id of new policy created
	 * @param description of new policy
	 * @deprecated
	 *
	 * @return String policy as String instance.
	 */
int createPolicy(const char* __policyID,const char* __description, char** returnValue);


	/**
	 * Creates an empty policy containing target as provided.  
	 * 
	 * @param policyID policy's id
	 * @param description description of the policy
	 * @param target target upon whom this policy apply
	 * @return policy created
	 * @throws Exception 
	 */
int createPolicyWithTarget(const char* __policyID,const char* __description,const char* __target, char** returnValue);


	/**
	 * Policy with policyId will be removed from policy storage.
	 *
	 * @param policyId
	 * @return true on success
	 * @param userCtx certificate of the subject executing this call.
	 * @throws instance of Exception, error from {@link PolicyFactory#removePolicy(String)}.
	 */
int removePolicy(const char* __policyId,const char* __userCtx, char* returnValue);


	/**
	 * Note that list of all policies can be very large. See also {@link VOPS#listFilteredPolicy(String)}
	 * and {@link VOPS#listPolicy(String)}.
	 *  
	 * @param userCtx certificate of the subject executing this call.
	 * @return ArrayList of XACML policies as ArrayList of strings.
	 */
int listPolicies(const char* __userCtx, void* returnValue);


	/**
	 * Lists specific policy with policyId. Returns XACML policy as String object.
	 *
	 * @param policy id.
	 * @param userCtx certificate of the subject executing this call.
	 * @return XACML policy as a string.
	 */
int listPolicy(const char* __policyId,const char* __userCtx, char** returnValue);


	/**
	 * Adds XACML policy into policy storage.
	 *
	 * @param Policy in XML passed as string (XACML format).
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success. 
	 */
int addPolicy(const char* __xacmlPolicy,const char* __userCtx, char* returnValue);


	/**
	 * Adds rule which is passed as XML string to the policy identified by 
	 * policyId. Returns rule as String object. See also 
	 * {@link VOPS#addXACMLRule(String, String)}, where rule is passed in a 
	 * XACML format. 
	 * 
	 * @param ruleXML description of the rule. 
	 * @param policyId defines destination policy.
	 * @return XACML rule as a string.  
	 */
int addRule(const char* __ruleXML,const char* __policyId, char** returnValue);


	/**
	 * Adds rule which is passed as XML string in XACML format to the policy
	 * identified by policyId.Returns rule created as String object.  
	 * 
	 * @param ruleXACML XACML string presenting rule to be added to policy
	 * @param policyId identifies to which type of policies are we this rule.
	 * @param userCtx certificate of the subject executing this call.  
	 * @return rule in a XACML format as a String. 
	 */
int addXACMLRule(const char* __ruleXACML,const char* __policyId,const char* __userCtx, char** returnValue);


	/**
	 * Returns a policy comprising rules which comply with the request passed as
	 * an argument.  
	 * 
	 * @param xacmlRequest XACML request which is applied to policies residing in 
	 * PolicyFactory.
	 *  
	 * @return XACML Policy comprising rules, which apply to xacmlReqeust. 
	 */
int listFilteredPolicy(const char* __xacmlRequest, char** returnValue);


	/**
	 * Lists policies which apply to certificate provided.
	 * 
	 * @param targetSubject
	 * @return
	 */
int listFilteredPolicyCert(const char* __targetSubjectCertificate, char** returnValue);



int generateRequest(const char* __targetSubjectCertificate, char** returnValue);


	/**
	 * Method constructs XACML request from XML passed as request and
	 * applies xacml request against policies stored in policy storage @see PolicyFactory.
	 *  
	 * @param request String presenting XACMl request
	 * @return XML file presenting XACML reply (see XACML 1.0 core specifications
	 * for more details   ). 
	 */
int evaluateRequest(const char* __xacmlRequest, char** returnValue);


	/**
	 * Removes rule from policy with specified policyId. 
	 *
	 * @param ruleId Rule which will be removed from policy.
	 * @param policyId policy from which this rule is removed.
	 * @param userCtx certificate of the subject executing this call. 
	 * @return true on success.
	 */
int removeRuleFromPolicy(const char* __ruleId,const char* __policyId,const char* __userCtx, char* returnValue);


	/**
	 * Writes back policies from policy storage in PolicyFactory on to local disk. 
	 * 
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success.
	 */
int writeBack(const char* __userCtx, char* returnValue);


	/**
	 * Reloads all policies stored in policy storage.
	 * @param userCtx certificate of the subject executing this call.
	 * @return true on success
	 */
int reloadVOPS(const char* __userCtx, char* returnValue);


	/**
	 * Return list of filtered addresses where execution is permitted.
	 * @return list of addressses
	 * @deprecated
	 */
int getFilteredResources( void* returnValue);


	/**
	 * This registers VOPS to global VOPS service where decisions are made. 
	 * 
	 * @param address VOPS address to register
	 * @param certificate of the VOPS
	 * 
	 * @return
	 */
int registerVOPSToGlobalVOPS(const char* __address,const char* __certificate, char* returnValue);


	/**
	 * These attributes can be used in XACML policies, requests and rules. 
	 * 
	 * @return an ArrayList containing subject attributes. 
	 */
int getSubjectAttributes( void* returnValue);


	/**
	 * These attributes can be used in XACML policies, requests and rules.
	 * 
	 * @return an ArrayList containing resource attributes.
	 */
int getResourceAttributes( void* returnValue);


	/**
	 * These attributes can be used in XACML policies, requests and rules.
	 * 
	 * @return an ArrayList containing action attributes.
	 */
int getActionAttributes( void* returnValue);


#ifdef __cplusplus 
 } 
 #endif 
