/*  Copyright (c) 2008 Telefónica Investigación y Desarrollo SAU.

This file is part of LinuxMD-XOS. LinuxMD-XOS is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 ** @file  xos_getcred.h
 ** @brief libxosgetcred interface
 **
 ** This file should be included by any application that use libxosgetcred
 **/

#ifndef XOS_GETCRED_H
#define XOS_GETCRED_H

/** @brief get credential from credstore if available; otherwise invoke
 ** runcredagent to get it.
 ** @param[in] configuration_name the configuration name used to obtain the
 **            credential. If NULL retrieve the configuration present in
 **            credstore (if any) without check the configuration name. If
 **            NULL and credstore is empty, use "default" configuration.
 ** @retval string with the credential
 ** @retval NULL if failure
 **
 ** This function is used to provide a single sign-on to applications. When
 ** a application invokes this function, library first searchs the credential
 ** in credstore. If credstore is not empty, checks if configuration_name is
 ** the same configuration_name that was specified when current stored
 ** credential was obtained. If names matchs, returns the credential. Also
 ** return the stored credential if configuration_name is NULL.
 ** If credstore is empty or configuration names don't match, runcredagent
 ** is called with the configuration name as parameter. Runcredagent use
 ** configuration_name to known how to get the credential (if NULL, use
 ** "default" configuration). Runcredagent store the obtained credential in
 ** credstore and this function returns it to application.
 **
 ** Runcredagent executable is not part of libxos_getcred. Package startxtreemos
 ** provides a implementation of runcredagent but other implementations are
 ** possible. These are the requirements that runcredagent must obey:
 **  -Application receives two parameters: "-l" and "-c <configuration_name>"
 **  -Application must store the retrieved credential using libcredstore

 ** A credential may be present in credstore after a previous call to
 ** runcredagent invoked from this function, but other cause is that user
 ** manually invoked startxtreemos or startxtreemos-ams. A third possibility is
 ** that user invokes libcredstore or some of its utilities to store a arbitrary
 ** content in credstore. This implies that is not guaranteed that credential
 ** stored in credstore is good in all cases (e.g. a malicious Trojan may
 ** replace the credential if running with the same UID than user applications).
 ** 
 ** In XtreemOS, credential is expected to be a private key and X.509
 ** certificate in PEM format, but this function does not check it, because
 ** other applications may use the library to use other type of credentials
 ** (e.g. passphrases).
 **/

extern char *xos_getcred(char *configuration_name);

/** @brief clear and delete credential; optionally, purge delete from credstore
 ** 
 ** @param[in] credential the credential as returned by xos_getcred. This
 **          parameter may be NULL, if the only purpose of the call is clear
 **          the credstore.
 ** @param[in] clear_credstore if this parameter is not zero, then clear
 **          current content of credstore.
 ** @retval This function does not return any value.
 **
 ** This auxiliar function fill with zeroes the credential and then delete it.
 ** This function also allows to delete the current credential stored in
 ** credstore (this functionality does not need the parameter credential, 
 ** because credstore only store one credential)
 **/
extern void xos_delcred(char *credential,char clear_credstore);
#endif
